
#include "bzfsAPI.h"
#include <string>
#include <vector>
#include "ShotTracker.h"

_ShotTracker ShotTracker = _ShotTracker();

Shot::Shot(int _owner, int _shotID, float _lifetime)
: owner(_owner)
, shotID(_shotID)
, lifetime(_lifetime)
, isEmpty(false)
, timeAdded(bz_getCurrentTime())
{
}

void _ShotTracker::add(int owner, int shotID, float lifetime)
{
	if(!checkShot(shotID).isEmpty)
	{
		for(std::vector<Shot>::iterator it = shotlist.begin(); it != shotlist.end(); it++)
		{
			if(it->shotID == shotID)
			{
				(*it) = Shot(owner, shotID, lifetime);
			}
		}
	}
	else
	{
		shotlist.push_back(Shot(owner, shotID, lifetime));
	}
}

Shot _ShotTracker::findShot(int shotID)
{
	return checkShot(shotID);
}

std::vector<Shot> _ShotTracker::findShots(int owner)
{
	std::vector<Shot> result;

	for(std::vector<Shot>::iterator it = shotlist.begin(); it != shotlist.end(); it++)
	{
		if(it->owner == owner)
		{
			Shot shot = checkShot(0, int(shotlist.begin() - it));

			if(!shot.isEmpty)
				result.push_back(shot);
		}
	}

	return result;
}

int _ShotTracker::getUniqueShotID()
{
	nextShot += 1;

	if(nextShot > 30000)
		nextShot -= 30000;

	return nextShot;
}

void _ShotTracker::remove(int shotID, int index)
{
	if(index != -1)
	{
		std::vector<Shot>::iterator it = shotlist.begin();
		std::advance<std::vector<Shot>::iterator, int>(it, index);
		shotlist.erase(it);
#ifdef DEBUG
		bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Removing shot %i", (shotlist.begin() + index)->shotID);
#endif
		return;
	}

	for(std::vector<Shot>::iterator it = shotlist.begin(); it != shotlist.end(); it++)
	{
		if(it->shotID == shotID)
		{
			shotlist.erase(it);
#ifdef DEBUG
			bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Removing shot %i", it->shotID);
#endif
			return;
		}
	}
}

Shot _ShotTracker::checkShot(int shotID, int index)
{
	double time = bz_getCurrentTime();

	if(index != -1)
	{
		Shot* shot = &shotlist[index];

		if((shot->timeAdded + shot->lifetime) < time) // It's gone
		{
			remove(0, index);
			return Shot();
		}
		else
		{
			return Shot(shot->owner, shot->shotID, shot->lifetime);
		}
	}

	for(std::vector<Shot>::iterator it = shotlist.begin(); it != shotlist.end(); it++)
	{
		if(it->shotID == shotID)
		{
			if((it->timeAdded + it->lifetime) < time) // It's gone
			{
				remove(0, int(it - shotlist.begin()));
				return Shot();
			}
			else
			{
				return Shot(it->owner, it->shotID, it->lifetime);
			}
		}
	}

	return Shot();
}

void _ShotTracker::purge()
{
	for(unsigned int i = 0; i < shotlist.size(); i++)
		checkShot(0, i);
}