
#include <string>
#include <vector>
#include "bzfsAPI.h"
#include "common.h"
#include "TurretEjector.h"
#include "TurretZone.h"

TurretEjector::TurretEjector(std::vector<TurretZone> *_zoneList)
: zoneList(_zoneList)
{
}

void TurretEjector::checkPosition(bz_PlayerRecord *player){

	Point playerPos = Point(player->pos[0], player->pos[1], player->pos[2]);

	Point dummy = Point();
	bool inside = obsticle.inside(playerPos, dummy);

	if(!inside)
		return;

	for(std::vector<unsigned int>::iterator it = zones.begin(); it != zones.end(); it++)
	{
		TurretZone* zone = &zoneList->at(*it);
		int playerToKill = zone->getOccupent();

		if(playerToKill != -1 &&
			playerToKill != player->playerID)
		{
			if(!ejectTeammates)
			{
				bz_PlayerRecord *playerKilled = bz_getPlayerByIndex(playerToKill);
				if(playerKilled->team != eRogueTeam && playerKilled->team == player->team)
				{
					bz_freePlayerRecord(playerKilled);
					continue;
				}
				bz_freePlayerRecord(playerKilled);
			}

			bz_killPlayer(playerToKill, false, player->playerID);
			bz_sendTextMessagef(BZ_SERVER, playerToKill, "You were ejected out of the turret by %s.", getCallsign(player->playerID).c_str());
			bz_sendTextMessagef(BZ_SERVER, player->playerID, "You ejected %s out of the turret.", getCallsign(playerToKill).c_str());
		}
	}

}

bool TurretEjector::parse(bz_CustomMapObjectInfo *data){

	Point pos;
	Point size;
	float radius;
	float height;
	float rotation;
	std::vector<std::string> tZones;

	bool handled [7] = {false};

	for(unsigned int i = 0; i < data->data.size(); i++){

		std::string line = data->data.get(i).c_str();
		bzAPIStringList *nubs = bz_newStringList();
		nubs->tokenize(line.c_str(), " ", 0, true);

		if(nubs->size() > 0){

			std::string key = bz_tolower(nubs->get(0).c_str());

			if(key == "position"){
				if(!handled[0]){
					if(Parser::position(nubs, pos))
						handled[0] = true;
				}
				else
					sendWarningMessage("Already defined a position, skipping.");
			}
			else if(key == "size"){
				if(!handled[1]){
					if(Parser::size(nubs, size))
						handled[1] = true;
				}
				else
					sendWarningMessage("Already defined a size, skipping.");
			}
			else if(key == "addzone"){
				if(nubs->size() == 2){
					tZones.push_back((std::string)nubs->get(1).c_str());
					handled[2] = true;
				}
				else
					sendErrorMessage((std::string)bz_format("addZone takes 1 argument, %i provided.", nubs->size() - 1));
			}
			else if(key == "ejectteammates"){
				if(!handled[3]){
					if(nubs->size() == 1){
						ejectTeammates = true;
						handled[3] = true;
					}
					else
						sendErrorMessage((std::string)bz_format("EjectTeammtes takes 0 arguments, %i provided.", nubs->size() - 1));
				}
				else
					sendWarningMessage("Already defined ejectTeammates, skipping.");
			}
			else if(key == "rotation"){
				if(!handled[4]){
					if(Parser::rotation(nubs, rotation))
						handled[4] = true;
				}
				else
					sendWarningMessage("Already defined a rotation, skipping.");
			}
			else if(key == "radius"){
				if(!handled[5]){
					if(Parser::radius(nubs, radius))
						handled[5] = true;
				}
				else
					sendWarningMessage("Already defined a radius, skipping.");
			}
			else if(key == "height"){
				if(!handled[6]){
					if(Parser::height(nubs, height))
						handled[6] = true;
				}
				else
					sendWarningMessage("Already defined a height, skipping.");
			}
			else
				sendWarningMessage((std::string)bz_format("Unknown property %s, skipping."));

			bz_deleteStringList(nubs);
		}
	}

	ObsticleType type = eBox;

	if((!handled[1] && !handled[5] && !handled[6]) || (handled[1] && (handled[5] || handled[6]))){
		sendErrorMessage("Unknown object type; specify either size or radius and height.");
		sendErrorMessage("Discarding object.");
		return false;
	}
	if(handled[2]){
		sendDebugMessage("Determined this to be a box.");
		type = eBox;
	}
	if(handled[5] || handled[6]){
		sendDebugMessage("Determined this to be a cylinder.");
		type = eCylinder;
	}
	if(!handled[0]){
		sendErrorMessage("Position not defined, discarding object.");
		return false;
	}
	if(!handled[1] && type == eBox){
		sendErrorMessage("Size not defined, discarding object.");
		return false;
	}
	if(!handled[2]){
		sendErrorMessage("No zones specified for this ejector, discarding object.");
		return false;
	}
	if(!handled[3]){
		sendDebugMessage("EjectTeammtes not defined, defaulting to no.");
		ejectTeammates = false;
	}
	if(!handled[4]){
		sendDebugMessage("Rotation not defined, defaulting to 0.");
		rotation = 0;
	}
	if(!handled[5] && type == eCylinder){
		sendErrorMessage("Radius not definied, discarding object.");
		return false;
	}
	if(!handled[6] && type == eCylinder){
		sendErrorMessage("Height not definied, discarding object.");
		return false;
	}

	for(std::vector<std::string>::iterator it = tZones.begin(); it != tZones.end(); it++)
	{
		bool found = false;

		for(unsigned int i = 0; i < zoneList->size(); i++)
		{
			if(*it == zoneList->at(i).getName())
			{
				zones.push_back(i);
				found = true;
			}
		}

		if(!found)
			sendWarningMessage((std::string)bz_format("Unable to find zone '%s'.", it->c_str()));
	}

	if(zones.size() == 0)
	{
		sendErrorMessage("No zones assigned to this ejector, discarding object.");
		return false;
	}

	if(type == eBox)
		obsticle = Obsticle(Box(pos, size, rotation));

	else
		obsticle = Obsticle(Cylinder(pos, radius, height));

	return true;
}