
#include <string>
#include <vector>
#include <stdio.h> // For stupid linux
#include <stdlib.h>
#include "bzfsAPI.h"
#include "TurretWeapon.h"
#include "common.h"
#include "ShotTracker.h"

extern _ShotTracker ShotTracker;

TurretWeapon::TurretWeapon()
{
	lastTime = 0.0;
}

void TurretWeapon::fire(bz_PlayerRecord *player, Obsticle obsticle, Point newPos){

	float rotation = (player->rot * RAD_TO_DEG) + 270;
	rotation = rotation >= 360? 360 - (rotation - 360): 360 - rotation;

	if(angleExtent[0] > rotation || rotation > angleExtent[1])
		return;

	double time = bz_getCurrentTime();
	double timeDifference = time - lastTime;

	if(timeDifference < (1.0/frequency))
		return;

	Point playerPos = Point(player->pos[0], player->pos[1], player->pos[2]);
	float tilt = 0;

	if(obsticle.type == eBox)
	{
		Box box = obsticle.box;
#ifdef DEBUG
		//bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Y Pos: %f.2", playerPos.y);
#endif
		//       ------- Distance to Upper Y Extent -------
		//       ----- Upper Y Extent ------                    -Length of Y -
		tilt = ((box.position.y + box.size.y) - newPos.y) / (box.size.y * 2);
	}

	else
	{
		Cylinder cylinder = obsticle.cylinder;
		tilt = newPos.x / cylinder.radius;
	}

#ifdef DEBUG
	//bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "TiltRatio: %f", tilt);
#endif

	tilt = (tilt * (tiltExtent[1] - tiltExtent[0]) + tiltExtent[0]) * DEG_TO_RAD;

	double _lifetime = float(lifetime);

	if(lifetime == -1){
		if(type == "GM")
			_lifetime = bz_getBZDBDouble("_shotSpeed") * bz_getBZDBDouble("_gmAdLife");
		else if(type == "SW")
			_lifetime = bz_getBZDBDouble("_reloadTime") * bz_getBZDBDouble("_shockAdLife");
		else if(type == "MG")
			_lifetime = bz_getBZDBDouble("_reloadTime") * bz_getBZDBDouble("_mGunAdLife");
		else if(type == "L")
			_lifetime = bz_getBZDBDouble("_reloadTime") * bz_getBZDBDouble("_laserAdLife");
		else if(type == "TH")
			_lifetime = bz_getBZDBDouble("_reloadTime") * bz_getBZDBDouble("_thiefAdLife");
		else
			_lifetime = bz_getBZDBDouble("_reloadTime");
	}

	float tPos [] = {pos.x, pos.y, pos.z};
	int shotID = ShotTracker.getUniqueShotID();

#ifdef DEBUG
	//bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Type: %s, Rotation %f, Tilt %f, Lifetime %f, ShotID %i", type.c_str(), rotation, tilt, _lifetime, shotID);
#endif

	bz_fireWorldWep(type.c_str(),
		float(_lifetime),
		BZ_SERVER,
		tPos,
		tilt,
		player->rot,
		shotID,
		0);

	ShotTracker.add(player->playerID, shotID, float(_lifetime));

	lastTime = time;
}

bool TurretWeapon::parse(bz_CustomMapObjectInfo *data){

	bool handled [7] = {false};

	for(unsigned int i = 0; i < data->data.size(); i++){

		std::string line = data->data.get(i).c_str();
		bzAPIStringList *nubs = bz_newStringList();
		nubs->tokenize(line.c_str(), " ", 0, true);

		if(nubs->size() > 0){

			std::string key = bz_tolower(nubs->get(0).c_str());

			if(key == "name"){
				if(nubs->size() == 2){
					if(!handled[0]){
						name = nubs->get(1).c_str();
						handled[0] = true;
					}
					else
						sendWarningMessage("Already defined a name, skipping.");
				}
				else
					sendErrorMessage((std::string)bz_format("Name takes 1 argument, %i provided.", nubs->size() - 1));
			}
			else if(key == "position"){
				if(!handled[1]){
					if(Parser::position(nubs, pos))
						handled[1] = true;
				}
				else
					sendWarningMessage("Already defined a position, skipping.");
			}
			else if(key == "type"){
				if(nubs->size() == 2){
					if(!handled[2]){
						type = bz_toupper(nubs->get(1).c_str());
						handled[2] = true;
					}
					else
						sendWarningMessage("Already defined a type, skipping.");
				}
				else
					sendErrorMessage((std::string)bz_format("Type takes 1 argument, %i provided.", nubs->size() - 1));
			}
			else if(key == "lifetime"){
				if(nubs->size() == 2){
					if(!handled[3]){
						if(!isNumber(nubs->get(1))){
							sendErrorMessage("Argument for lifetime is not a number.");
						}
						else{
							lifetime = (float)atof(nubs->get(1).c_str());
							handled[3] = true;
						}
					}
					else
						sendWarningMessage("Already defined a lifetime, skipping.");
				}
				else
					sendErrorMessage((std::string)bz_format("Lifetime takes 1 argument, %i provided.", nubs->size() - 1));
			}
			else if(key == "frequency"){
				if(nubs->size() == 2){
					if(!handled[4]){
						if(!isNumber(nubs->get(1))){
							sendErrorMessage("Argument for frequency is not a number.");
						}
						else{
							frequency = atoi(nubs->get(1).c_str());
							handled[4] = true;
						}
					}
					else
						sendWarningMessage("Already defined a frequency, skipping.");
				}
				else
					sendErrorMessage((std::string)bz_format("Frequency takes 1 argument, %i provided.", nubs->size() - 1));
			}
			else if(key == "tiltextent"){
				if(nubs->size() == 3){
					if(!handled[5]){
						if(!isNumber(nubs->get(1))){
							sendErrorMessage("First argument of tiltExtent is not a number.");
						}
						else if(!isNumber(nubs->get(2))){
							sendErrorMessage("Second argument of tiltExtent is not a number.");
						}
						else{
							tiltExtent.push_back((float)atof(nubs->get(1).c_str()));
							tiltExtent.push_back((float)atof(nubs->get(2).c_str()));
							handled[5] = true;
						}
					}
					else
						sendWarningMessage("TiltExtent already defined, skipping.");
				}
				else
					sendErrorMessage((std::string)bz_format("TiltExtent takes 2 arguments, %i provided.", nubs->size() - 1));
			}
			else if(key == "angleextent"){
				if(nubs->size() == 3){
					if(!handled[6]){
						if(!isNumber(nubs->get(1))){
							sendErrorMessage("First argument of angleExtent is not a number.");
						}
						else if(!isNumber(nubs->get(2))){
							sendErrorMessage("Second argument of angleExtent is not a number.");
						}
						else{
							angleExtent.push_back((float)atof(nubs->get(1).c_str()));
							angleExtent.push_back((float)atof(nubs->get(2).c_str()));
							handled[6] = true;
						}
					}
					else
						sendWarningMessage("AngleExtent alread defined, skipping.");
				}
				else
					sendErrorMessage((std::string)bz_format("AngleExtent takes 2 arguments, %i provided.", nubs->size() - 1));
			}
			else
				sendWarningMessage((std::string)bz_format("Unknown property %s, skipping.", key.c_str()));

			bz_deleteStringList(nubs);
		}
	}
	if(!handled[0]){
		sendErrorMessage("Name not defined, discarding object.");
		return false;
	}
	if(!handled[1]){
		sendErrorMessage("Position not defined, discarding object.");
		return false;
	}
	if(!handled[2]){
		sendDebugMessage("Type not defined, defaulting to normal shot.");
		type = "US";
	}
	if(!handled[3]){
		sendDebugMessage("Lifetime not defined, defaulting to BZDB value.");
		lifetime = -1;
	}
	if(!handled[4]){
		sendDebugMessage("Frequency not defined, defaulting to 5.");
		frequency = 5;
	}
	if(!handled[5]){
		sendDebugMessage("TiltExtent not defined, defaulting to 0 and 360.");
		tiltExtent.push_back(0.0);
		tiltExtent.push_back(360.0);
	}
	if(!handled[6]){
		sendDebugMessage("AngleExtent not defined, defaulting to 0 and 360.");
		angleExtent.push_back(0.0);
		angleExtent.push_back(360.0);
	}

	if(frequency <= 0)
		frequency = 1;

	return true;
}