#include <string>
#include <vector>
#include <stdio.h> // For stupid linux
#include <stdlib.h>
#include "common.h"
#include "bzfsAPI.h"
#include "TurretWeapon.h"
#include "TurretZone.h"

TurretZone::TurretZone(std::vector<TurretWeapon> *_weaponList)
: weaponList(_weaponList)
, playerInside(-1)
{
}

void TurretZone::checkPosition(bz_PlayerRecord *player){


	Point newPos = Point();
	bool inside = obsticle.inside(Point(player->pos[0], player->pos[1], player->pos[2]), newPos);

	if(!inside){
		if(playerInside == player->playerID)
			playerInside = -1;
		return;
	}

	if(playerInside != -1 && player->playerID != playerInside){
		bz_killPlayer(player->playerID, false, player->playerID);
		bz_sendTextMessage(player->playerID, player->playerID, "Only one person in the turret at a time!");
		return;
	}

	if(playerInside == -1)
		playerInside = player->playerID;

	for(std::vector<unsigned int>::iterator it = weapons.begin(); it != weapons.end(); it++)
		weaponList->at(*it).fire(player, obsticle, newPos);
}

bool TurretZone::parse(bz_CustomMapObjectInfo *data){

	Point pos;
	Point size;
	float rotation;
	float radius;
	float height;
	std::vector<std::string> tWeapons;

	bool handled [7] = {false};

	for(unsigned int i = 0; i < data->data.size(); i++){

		std::string line = data->data.get(i).c_str();
		bzAPIStringList *nubs = bz_newStringList();
		nubs->tokenize(line.c_str(), " ", 0, true);

		if(nubs->size() > 0){

			std::string key = bz_tolower(nubs->get(0).c_str());

			if(key == "name"){
				if(nubs->size() == 2){
					if(!handled[0]){
						name = nubs->get(1).c_str();
						handled[0] = true;
					}
					else
						sendWarningMessage("Already defined a name, skipping.");
				}
				else
					sendErrorMessage((std::string)bz_format("Name takes 1 argument, %i provided.", nubs->size() - 1));
			}
			else if(key == "position"){
				if(!handled[1]){
					if(Parser::position(nubs, pos))
						handled[1] = true;
				}
				else
					sendWarningMessage("Already defined a position, skipping.");
			}
			else if(key == "size"){
				if(!handled[2]){
					if(Parser::size(nubs, size))
						handled[2] = true;
				}
				else
					sendWarningMessage("Already defined a size, skipping.");
			}
			else if(key == "addweapon"){
				if(nubs->size() == 2){
					tWeapons.push_back((std::string)nubs->get(1).c_str());
					handled[3] = true;
				}
				else
					sendErrorMessage((std::string)bz_format("AddWeapon takes 1 argument, %i provided.", nubs->size() - 1));
			}
			else if(key == "rotation"){
				if(!handled[4]){
					if(Parser::rotation(nubs, rotation))
						handled[4] = true;
				}
				else
					sendWarningMessage("Already defined a rotation, skipping.");
			}
			else if(key == "radius"){
				if(!handled[5]){
					if(Parser::radius(nubs, radius))
						handled[5] = true;
				}
				else
					sendWarningMessage("Already defined a radius, skipping.");
			}
			else if(key == "height"){
				if(!handled[6]){
					if(Parser::height(nubs, height))
						handled[6] = true;
				}
				else
					sendWarningMessage("Already defined a height, skipping.");
			}
			else
				sendWarningMessage((std::string)bz_format("Unknown property %s, skipping.", nubs->get(0).c_str()));

			bz_deleteStringList(nubs);
		}
	}

	ObsticleType type = eBox;

	if((!handled[2] && !handled[5] && !handled[6]) || (handled[2] && (handled[5] || handled[6]))){
		sendErrorMessage("Unknown object type; specify either size or radius and height.");
		sendErrorMessage("Discarding object.");
		return false;
	}
	if(handled[2]){
		sendDebugMessage("Determined this to be a box.");
		type = eBox;
	}
	if(handled[5] || handled[6]){
		sendDebugMessage("Determined this to be a cylinder.");
		type = eCylinder;
	}
	if(!handled[0]){
		sendErrorMessage("Name not defined, discarding object.");
		return false;
	}
	if(!handled[1]){
		sendErrorMessage("Position not defined, discarding object.");
		return false;
	}
	if(!handled[2] && type == eBox){
		sendErrorMessage("Size not defined, discarding object.");
		return false;
	}
	if(!handled[3]){
		sendWarningMessage("No weapons assigned to this zone.");
	}
	if(!handled[4] && type == eBox){
		sendDebugMessage("Rotation not defined, defaulting to 0.");
		rotation = 0;
	}
	if(!handled[5] && type == eCylinder){
		sendErrorMessage("Radius not defined, discarding object.");
		return false;
	}
	if(!handled[6] && type == eCylinder){
		sendErrorMessage("Height not defined, discarding object.");
		return false;
	}

	for(std::vector<std::string>::iterator it = tWeapons.begin(); it != tWeapons.end(); it++)
	{
		bool found = false;

		for(unsigned int i = 0; i < weaponList->size(); i++)
		{
			if(*it == weaponList->at(i).getName())
			{
				weapons.push_back(i);
				found = true;
			}
		}

		if(!found)
			sendWarningMessage((std::string)bz_format("Unable to find weapon '%s'.", it->c_str()));
	}

	if(weapons.size() == 0)
		sendWarningMessage("No weapons assigned to this zone.");

	if(type == eBox)
		obsticle = Obsticle(Box(pos, size, rotation));

	else
		obsticle = Obsticle(Cylinder(pos, radius, height));

	return true;
}